@@ -15,7 +15,9 @@
 #                  and http://forum.photome.de/viewtopic.php?f=2&t=353&p=742#p740
 #               7) Kai Lappalainen private communication
 #               8) http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,5223.0.html
-#               9) Iliah Borg private communication (LibRaw)
+#               9) Zilvinas Brobliauskas private communication
+#               10) Albert Shan private communication
+#               IB) Iliah Borg private communication (LibRaw)
 #               JD) Jens Duttke private communication
 #------------------------------------------------------------------------------
 
@@ -26,7 +28,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.53';
+$VERSION = '1.55';
 
 sub ProcessFujiDir($$$);
 sub ProcessFaceRec($$$);
@@ -152,6 +154,10 @@ my %faceCategories = (
             0x303 => 'B&W Green Filter', #PH/8
             0x310 => 'B&W Sepia', #PH (X100)
             0x400 => 'Low 2', #8 ("Color -2")
+            0x500 => 'Acros', #PH (X-Pro2)
+            0x501 => 'Acros Red Filter', #PH (X-Pro2)
+            0x502 => 'Acros Yellow Filter', #PH (X-Pro2)
+            0x503 => 'Acros Green Filter', #PH (X-Pro2)
             0x8000 => 'Film Simulation', #2
         },
     },
@@ -214,12 +220,17 @@ my %faceCategories = (
     0x1010 => {
         Name => 'FujiFlashMode',
         Writable => 'int16u',
+        PrintHex => 1,
         PrintConv => {
             0 => 'Auto',
             1 => 'On',
             2 => 'Off',
             3 => 'Red-eye reduction',
             4 => 'External', #JD
+            16 => 'Commander',
+            0xa920 => '1st Curtain (front)', #10 (EF-X500 flash)
+            0xc920 => '2nd Curtain (rear)', #10
+            0xe920 => 'High Speed Sync (HSS)', #10
         },
     },
     0x1011 => {
@@ -350,6 +361,12 @@ my %faceCategories = (
             32 => 'Soft',
         },
     },
+    0x1044 => { #forum7668
+        Name => 'DigitalZoom',
+        Writable => 'int32u',
+        ValueConv => '$val / 8',
+        ValueConvInv => '$val * 8',
+    },
     0x1050 => { #forum6109
         Name => 'ShutterType',
         Writable => 'int16u',
@@ -375,6 +392,20 @@ my %faceCategories = (
     # 0x1150 - Pro Low-light - val=1; Pro Focus - val=2 (ref 7)
     # 0x1151 - Pro Low-light - val=4 (number of pictures taken?); Pro Focus - val=2,3 (ref 7)
     # 0x1152 - Pro Low-light - val=1,3,4 (stacked pictures used?); Pro Focus - val=1,2 (ref 7)
+    0x1153 => { #forum7668
+        Name => 'PanoramaAngle',
+        Writable => 'int16u',
+    },
+    0x1154 => { #forum7668
+        Name => 'PanoramaDirection',
+        Writable => 'int16u',
+        PrintConv => {
+            1 => 'Right',
+            2 => 'Up',
+            3 => 'Left',
+            4 => 'Down',
+        },
+    },
     0x1201 => { #forum6109
         Name => 'AdvancedFilter',
         Writable => 'int32u',
@@ -449,12 +480,12 @@ my %faceCategories = (
         Writable => 'int16u',
         PrintHex => 1,
         PrintConv => {
-            0x000 => 'F0/Standard (Provia)',
+            0x000 => 'F0/Standard (Provia)', # X-Pro2 "Provia/Standard"
             0x100 => 'F1/Studio Portrait',
             0x110 => 'F1a/Studio Portrait Enhanced Saturation',
-            0x120 => 'F1b/Studio Portrait Smooth Skin Tone (Astia)',
+            0x120 => 'F1b/Studio Portrait Smooth Skin Tone (Astia)', # X-Pro2 "Astia/Soft"
             0x130 => 'F1c/Studio Portrait Increased Sharpness',
-            0x200 => 'F2/Fujichrome (Velvia)',
+            0x200 => 'F2/Fujichrome (Velvia)', # X-Pro2 "Velvia/Vivid"
             0x300 => 'F3/Studio Portrait Ex',
             0x400 => 'F4/Velvia',
             0x500 => 'Pro Neg. Std', #PH (X-Pro1)
@@ -692,60 +723,71 @@ my %faceCategories = (
         Count => 36,
         PrintConv => '$val =~ tr/012 /RGB/d; join " ", $val =~ /....../g',
     },
-    0x2000 => { #9
+    0x2000 => { #IB
         Name => 'WB_GRGBLevelsAuto',
         Format => 'int16u',
         Count => 4, # (ignore the duplicate values)
     },
-    0x2100 => { #9
+    0x2100 => { #IB
         Name => 'WB_GRGBLevelsDaylight',
         Format => 'int16u',
         Count => 4,
     },
-    0x2200 => { #9
+    0x2200 => { #IB
         Name => 'WB_GRGBLevelsCloudy',
         Format => 'int16u',
         Count => 4,
     },
-    0x2300 => { #9
+    0x2300 => { #IB
         Name => 'WB_GRGBLevelsDaylightFluor',
         Format => 'int16u',
         Count => 4,
     },
-    0x2301 => { #9
+    0x2301 => { #IB
         Name => 'WB_GRGBLevelsDayWhiteFluor',
         Format => 'int16u',
         Count => 4,
     },
-    0x2302 => { #9
+    0x2302 => { #IB
         Name => 'WB_GRGBLevelsWhiteFluorescent',
         Format => 'int16u',
         Count => 4,
     },
-    0x2310 => { #9
+    0x2310 => { #IB
         Name => 'WB_GRGBLevelsWarmWhiteFluor',
         Format => 'int16u',
         Count => 4,
     },
-    0x2311 => { #9
+    0x2311 => { #IB
         Name => 'WB_GRGBLevelsLivingRoomWarmWhiteFluor',
         Format => 'int16u',
         Count => 4,
     },
-    0x2400 => { #9
+    0x2400 => { #IB
         Name => 'WB_GRGBLevelsTungsten',
         Format => 'int16u',
         Count => 4,
     },
-    # 0x2f00 => WB_GRGBLevelsCustom: int32u count, then count * (int16u GRGBGRGB), ref 9
+    # 0x2f00 => WB_GRGBLevelsCustom: int32u count, then count * (int16u GRGBGRGB), ref IB
     0x2ff0 => {
         Name => 'WB_GRGBLevels',
         Format => 'int16u',
         Count => 4,
     },
+    0x9200 => { #Frank Markesteijn
+        Name => 'RelativeExposure',
+        Format => 'rational32s',
+        ValueConv => 'log($val) / log(2)',
+        ValueConvInv => 'exp($val * log(2))',
+        PrintConv => '$val ? sprintf("%+.1f",$val) : 0',
+        PrintConvInv => '$val',
+    },
+    # 0x9200 - relative exposure? (ref Frank Markesteijn)
     0x9650 => { #Frank Markesteijn
         Name => 'RawExposureBias',
         Format => 'rational32s',
+        PrintConv => '$val ? sprintf("%+.1f",$val) : 0',
+        PrintConvInv => '$val',
     },
     0xc000 => {
         Name => 'RAFData',
@@ -844,12 +886,13 @@ my %faceCategories = (
         OffsetPair => 0xf007,  # point to associated offsets
     },
     # 0xf009 - values: 0, 3
-    0xf00a => 'BlackLevel', #9
-    # 0xf00b ?
-    0xf00c => 'WB_GRBLevelsStandard', #9 (GRBXGRBX; X=17 is standard illuminant A, X=21 is D65)
-    0xf00d => 'WB_GRBLevelsAuto', #9
+    0xf00a => 'BlackLevel', #IB
+    0xf00b => 'GeometricDistortionParams', #9 (rational64s[23, 35 or 43])
+    0xf00c => 'WB_GRBLevelsStandard', #IB (GRBXGRBX; X=17 is standard illuminant A, X=21 is D65)
+    0xf00d => 'WB_GRBLevelsAuto', #IB
     0xf00e => 'WB_GRBLevels',
-    # 0xf00f ?
+    0xf00f => 'ChromaticAberrationParams', # (rational64s[23])
+    0xf010 => 'VignettingParams', #9 (rational64s[31 or 64])
 );
 
 # information found in FFMV atom of MOV videos
